@@ -60,7 +60,7 @@ from .colorramping import \
 #--------------------------------------------------------------------
 from . import commanding
 from .commanding import \
-      async,           \
+      async_,             \
       cls,                \
       delete,             \
       do,                 \
@@ -102,6 +102,7 @@ from .querying import \
       get_angle,          \
       get_area,           \
       get_assembly_ids,   \
+      get_bonds,          \
       get_chains,         \
       get_collada,        \
       get_color_index,    \
@@ -134,6 +135,7 @@ from .querying import \
       get_povray,         \
       get_raw_alignment,  \
       get_renderer,       \
+      get_selection_state,\
       get_symmetry,       \
       get_title,          \
       get_type,           \
@@ -166,6 +168,8 @@ from .exporting import \
       get_cifstr,         \
       get_session,        \
       get_fastastr,       \
+      multifilesave,      \
+      multifilenamegen,   \
       multisave,          \
       png,                \
       save               
@@ -179,6 +183,7 @@ from .editing import \
       alphatoall,         \
       attach,             \
       bond,               \
+      copy_to,            \
       cycle_valence,      \
       deprotect,          \
       drag,               \
@@ -235,6 +240,7 @@ from .editing import \
       translate_atom,     \
       unbond,             \
       undo,               \
+      uniquify,           \
       unpick,             \
       update,             \
       valence,            \
@@ -440,6 +446,9 @@ from .internal import      \
 from .util import \
       get_sasa_relative
 
+from .stereochemistry import \
+      assign_stereo
+
 #--------------------------------------------------------------------
 #from m4x import \
 #     metaphorics
