@@ -505,7 +505,7 @@ SEE ALSO
 
     ramp_new
         '''
-        return ramp_new(name, '', range, color, quiet=quiet, _self=cmd)
+        return _self.ramp_new(name, '', range, color, quiet=quiet)
 
     def isomesh(name, map, level=1.0, selection='', buffer=0.0,
                 state=1, carve=None, source_state=0, quiet=1, _self=cmd):
@@ -972,19 +972,16 @@ USAGE
                 object=name
             model = fragments.get(str(name))
             la = len(model.atom)
-            if la:
+            if la and int(origin):
                 position = _self.get_position()
                 for c in range(0,3):
                     mean_c = sum([a.coord[c] for a in model.atom]) / la
                     mean_c = position[c] - mean_c
                     for a in model.atom:
-                        _self._adjust_coord(a, c, mean_c)
+                        a.coord[c] += mean_c
             r = _self.load_model(model,str(object),quiet=quiet,zoom=zoom, _self=_self)
         except IOError:
-            print("Error: unable to load fragment '%s'." % name)
-        except:
-            traceback.print_exc()
-            print("Error: unable to load fragment '%s'." % name)         
+            raise pymol.CmdException("unable to load fragment '%s'." % name)
         if _self._raising(r,_self): raise pymol.CmdException                                    
         return r
 
@@ -1034,23 +1031,35 @@ SEE ALSO
             print(' Warning: properties are not supported in Open-Source PyMOL')
         # preprocess selection
         selection = selector.process(selection)
-        #      
+
+        # TODO is this too much convenience? 'extract' should be a simple boolean
+        if extract in (None, 0, '0', ''):
+            extract = ''
+        elif extract in (1, '1'):
+            extract = selection
+        else:
+            print(' Warning: non-boolean extract values are deprecated!')
+            extract = selector.process(extract)
+
+        if extract:
+            extract_sele = _self.get_unused_name('_extract')
+            _self.select(extract_sele, extract, 0)
+
         try:
             _self.lock(_self)
-            if name==None:
+            if not name:
                 name = _self.get_unused_name("obj")
             r = _cmd.create(_self._COb,str(name),"("+str(selection)+")",
                             int(source_state)-1,int(target_state)-1,
                             int(discrete),int(zoom),int(quiet),int(singletons))
         finally:
             _self.unlock(r,_self)
-        if not is_error(r): # temporary inefficient implementation
-            if extract not in (None, 0, '0'):
-                if extract not in (1, '1'):
-                    extract = selector.process(extract)
-                else:
-                    extract = selection
-                _self.remove("(("+extract+") in (%s)) and not (%s)"%(name,name))
+
+        if extract:
+            if not is_error(r):
+                _self.remove(extract_sele)
+            _self.delete(extract_sele)
+
         if _self._raising(r,_self): raise pymol.CmdException
         return r
 
@@ -1087,6 +1096,9 @@ SEE ALSO
     unquote_re = re.compile(r"r?('.*'|\".*\")$")
     
     def unquote(s):
+        if sys.version_info[0] > 2 and isinstance(s, bytes):
+            s = s.decode('utf-8', 'replace')
+
         s = str(s)
         if unquote_re.match(s):
             try:
