@@ -16,20 +16,20 @@ from __future__ import print_function, absolute_import
 
 if __name__=='pymol.exporting':
     import os
-    try:
+    import sys
+    if sys.version_info[0] == 2:
         import thread
-        import cPickle
-    except ImportError:
+    else:
         import _thread as thread
-        import pickle as cPickle
     from . import selector
     import re
     import copy
     
     import pymol
-    cmd = __import__("sys").modules["pymol.cmd"]
+    cmd = sys.modules["pymol.cmd"]
     from .cmd import _cmd,lock,unlock,Shortcut,QuietException
     from chempy import io
+    from chempy.pkl import cPickle
     from chempy.sdf import SDF,SDFRec
     from .cmd import _feedback,fb_module,fb_mask, \
                      DEFAULT_ERROR, DEFAULT_SUCCESS, _raising, is_ok, is_error, \
@@ -41,7 +41,7 @@ if __name__=='pymol.exporting':
         if thread.get_ident() == pymol.glutThread:
             r = _self._copy_image(_self,int(quiet))
         else:
-            r = _self.do('cmd._copy_image(quiet=%d,_self=cmd)'%int(quiet))
+            r = _self.do('cmd._copy_image(quiet=%d)'%int(quiet))
         if _self._raising(r,_self): raise QuietException         
         return r
 
@@ -121,20 +121,20 @@ PYMOL API
                 scene = scene.strip()
                 if not quiet:
                     print(" cache: optimizing scene '%s'."%scene)
-                cmd.scene(scene,animate=0)
-                cmd.rebuild()                
-                cmd.refresh()
+                _self.scene(scene,animate=0)
+                _self.rebuild()
+                _self.refresh()
             if len(cur_scene):
-                cmd.scene(cur_scene,animate=0)
+                _self.scene(cur_scene,animate=0)
             else:
                 scene_list = _self.get_scene_list()
                 if len(scene_list):
-                    cmd.scene(scene_list[0],animate=0)
+                    _self.scene(scene_list[0],animate=0)
                 else:
                     if not quiet:
                         print(" cache: no scenes defined -- optimizing current display.")
-                    cmd.rebuild() 
-                    cmd.refresh()
+                    _self.rebuild()
+                    _self.refresh()
             usage = _self._cache_purge(-1,_self=_self)
             if cache_mode:
                 _self.set('cache_mode',cache_mode)
@@ -171,33 +171,69 @@ PYMOL API
             'VAL' : 'V',
             'TRP' : 'W',
             'TYR' : 'Y',
+            # RNA
+            'A'   : 'A',
+            'U'   : 'U',
+            'G'   : 'G',
+            'C'   : 'C',
+            # DNA
+            'DA'  : 'A',
+            'DT'  : 'T',
+            'DG'  : 'G',
+            'DC'  : 'C',
             }
     
-    def get_fastastr(selection="all", state=-1, quiet=1, _self=cmd):
-        dict = { 'seq' : {} }
-        # we use (alt '' or alt 'A') because 'guide' picks up 
-        # non-canonical structures: eg, 1ejg has residue 22 as a SER and 
-        # PRO, which guide will report twice
-        _self.iterate("("+selection+") and polymer and name CA and alt +A",
-                    "seq[model]=seq.get(model,[]);seq[model].append(resn)",space=dict)
-        seq = dict['seq']
-        result = []
-        for obj in _self.get_names("objects",selection='('+selection+')'):
-            if obj in seq:
-                cur_seq = [_resn_to_aa.get(x,'?') for x in seq[obj]]
-                result.append(">%s"%obj)
-                cur_seq = ''.join(cur_seq)
-                while len(cur_seq):
-                    if len(cur_seq)>=70:
-                        result.append(cur_seq[0:70])
-                        cur_seq=cur_seq[70:]
-                    else:
-                        result.append(cur_seq)
-                        break
-        result = '\n'.join(result)
-        if len(result):
-            result = result + '\n'
-        return result
+    def get_fastastr(selection="all", state=-1, quiet=1, key='', _self=cmd):
+        '''
+DESCRIPTION
+
+    API only. Get protein and nucleic acid sequences in fasta format.
+
+    Used for saving:
+    PyMOL> save foo.fasta
+
+    New in PyMOL 2.2:
+    - chain specific keys (key argument)
+    - nucleic acid support
+
+ARGUMENTS
+
+    selection = str: atom selection (reduced to "guide & alt +A") {default: all}
+
+    state = int: (only used if state > 0)
+
+    quiet = 0/1: UNUSED
+
+    key = str: python expression {default: model + "_" + chain}
+    Use key=model to get the old (non-chain specific) behavior
+        '''
+        import textwrap
+        import collections
+
+        seq = collections.OrderedDict()
+        lines = []
+
+        selection = "(" + selection + ") & guide & alt +A"
+        if int(state) > 0:
+            selection += ' & state {}'.format(state)
+
+        if not key:
+            key = 'model + "_" + chain'
+            # for discrete objects: state specific keys
+            key += ' + (":{}".format(state) if state else "")'
+
+        _self.iterate(selection,
+            "seq.setdefault((" + key + "),[]).append(resn)",
+            space={'seq': seq, 'str': str})
+
+        for key, resn_list in seq.items():
+            cur_seq = ''.join(_resn_to_aa.get(x, '?') for x in resn_list)
+            lines.append(">{}".format(key))
+            lines.extend(textwrap.wrap(cur_seq, 70))
+
+        if lines:
+            lines.append('')  # final newline
+        return '\n'.join(lines)
 
     def get_pdbstr(selection="all", state=-1, ref='', ref_state=-1, quiet=1, _self=cmd):
         '''
@@ -220,15 +256,7 @@ NOTES
     if state is 0, then all states are saved.
     
     '''
-        r = DEFAULT_ERROR
-        try:
-            _self.lock(_self)   
-            r = _cmd.get_pdb(_self._COb,str(selection),int(state)-1,0,
-                             str(ref),int(ref_state),int(quiet))
-        finally:
-            _self.unlock(r,_self)
-        if _self._raising(r,_self): raise QuietException         
-        return r
+        return get_str('pdb', selection, state, ref, ref_state, -1, quiet, _self)
 
     def _get_dump_str(obj):
         if is_list(obj):
@@ -361,6 +389,34 @@ NOTES
         r = DEFAULT_SUCCESS
         cache = int(cache)
         compress = int(compress)
+        partial = int(partial)
+
+        pse_export_version = round(_self.get_setting_float('pse_export_version'), 4)
+        legacyscenes = (0 < pse_export_version < 1.76) and _self.get_scene_list()
+
+        if sys.version_info[0] > 2:
+            legacypickle = (0 < pse_export_version < 1.9)
+            if legacypickle:
+                print(' Warning: pse_export_version with Python 3 is experimental')
+            cPickle.configure_legacy_dump(legacypickle)
+
+        if legacyscenes:
+            _self.pymol._scene_dict = {}
+
+            scene_current_name = _self.get('scene_current_name')
+            tempname = '_scene_db96724c3cef00875c3bebb4f348f711'
+            _self.scene(tempname, 'store')
+
+            for name in legacyscenes:
+                _self.scene(name, 'recall', animate=0)
+                wizard = _self.get_wizard()
+                message = wizard.message if getattr(wizard, 'from_scene', 0) else None
+                pymol.viewing._legacy_scene(name, 'store', message, _self=_self)
+
+            _self.scene(tempname, 'recall', animate=0)
+            _self.scene(tempname, 'clear')
+            _self.set('scene_current_name', scene_current_name)
+
         if cache:
             cache_opt = int(_self.get('session_cache_optimize'))
             if cache != 0:
@@ -389,8 +445,15 @@ NOTES
                     traceback.print_exc()
                     print("Error: An error occurred when trying to generate session.")
                     print("Error: The resulting session file may be incomplete.")
+
+        if legacyscenes:
+            del session['moviescenes']
+            session['scene_dict'] = _self.pymol._scene_dict
+            session['scene_order'] = legacyscenes
+            pymol.viewing._legacy_scene('*', 'clear', _self=_self)
+            del _self.pymol._scene_dict
+
         if is_ok(r):
-            pse_export_version = round(_self.get_setting_float('pse_export_version'), 4)
             if pse_export_version > 0.0:
                 try:
                     _session_convert_legacy(session, pse_export_version, _self)
@@ -407,20 +470,26 @@ NOTES
             raise QuietException                  
         return r
         
-    def _unit2px(value, dpi):
+    def _unit2px(value, dpi, unit=''):
         '''API only. Returns pixel units given a string representation in other units'''
-        if isinstance(value, str):
+        if cmd.is_string(value):
             m = re.search(r'[a-z].*', value, re.I)
             if m:
-                if dpi <= 0:
-                    raise pymol.CmdException('need dpi if units are given')
                 value, unit = value[:m.start()], m.group(0).lower()
-                upi = {'in': 1.0, 'mm': 25.4, 'cm': 2.54}
-                if unit not in upi:
-                    raise pymol.CmdException('unknown unit, supported units are: ' +
-                            ', '.join(upi))
-                value = float(value) * dpi / upi[unit] + 0.5
-        return int(value)
+
+        if not unit or unit == 'px':
+            return int(value)
+
+        upi = {'in': 1.0, 'mm': 25.4, 'cm': 2.54}
+        if unit not in upi:
+            raise pymol.CmdException('unknown unit, supported units are: ' +
+                    ', '.join(upi))
+
+        if dpi < 1:
+            raise pymol.CmdException('dpi > 0 required with unit "%s" '
+                    '(hint: set the "image_dots_per_inch" setting)' % unit)
+
+        return float(value) * dpi / upi[unit] + 0.5
 
     def png(filename, width=0, height=0, dpi=-1.0, ray=0,
             quiet=1, prior=0, format=0, _self=cmd):
@@ -469,6 +538,10 @@ PYMOL API
         '''
         r = DEFAULT_ERROR
         prior = int(prior)
+
+        if format == 'png':
+            format = 0
+
         if prior: 
             # fetch the prior image, without doing any work (fast-path / non-GLUT thread-safe)
             r = _self._png(str(filename),0,0,float(dpi),0,int(quiet),1,
@@ -479,7 +552,7 @@ PYMOL API
         if not prior:
             dpi = float(dpi)
             if dpi < 0:
-                dpi = cmd.get_setting_float('image_dots_per_inch')
+                dpi = _self.get_setting_float('image_dots_per_inch')
             width = _unit2px(width, dpi)
             height = _unit2px(height, dpi)
 
@@ -508,33 +581,56 @@ PYMOL API
         '''
 DESCRIPTION
 
-    "multisave" is an unsupported command.
-    
+    "multisave" will save a multi-entry PDB file.
+
+    Every object in the given selection (pattern) will have a HEADER and a
+    CRYST (if symmetry is defined) record, and is terminated with END.
+    Loading such a multi-entry PDB file into PyMOL will load each entry
+    as a separate object.
+
+    This behavior is different to the "save" command, where a multi-object
+    selection is written "flat" to a PDB file, without HEADER or CRYST
+    records.
+
+ARGUMENTS
+
+    filename = string: file path to be written
+
+    pattern = str: atom selection (before 1.8.4: object name pattern)
+
+    state = int: object state (-1=current, 0=all) {default: -1}
+
+    append = 0/1: append to existing file {default: 0}
+
+    format = str: file format {default: guess from extension, or 'pdb'}
     '''
-        r = DEFAULT_ERROR
-        filename = _self.exp_path(filename)        
-        lc_filename = filename.lower()
-        if format=='':
-            # refactor following if/elif cascade 
-            # with a dictionary lookup
-            if re.search("\.pdb$|\.ent$",lc_filename):
-                format = 'pdb'
-            elif re.search("\.pmo$",lc_filename):
-                format = 'pmo'
-        if format == 'pdb':
-            ftype = loadable.pdb
-        elif format == 'pmo':
-            ftype = loadable.pmo
-        else:
-            ftype = loadable.pdb # default
-        try:
-            _self.lock(_self)
-            r = _cmd.multisave(_self._COb, str(filename), str(pattern),
-                               int(state)-1, int(append), int(ftype),int(quiet))
-        finally:
-            _self.unlock(r,_self)
-        if _self._raising(r,_self): raise QuietException
-        return r
+        from pymol.importing import filename_to_format
+        _, _, format_guessed, zipped = filename_to_format(filename)
+
+        if zipped:
+            raise pymol.CmdException(zipped + ' not supported with multisave')
+
+        if not format:
+            format = format_guessed or 'pdb'
+
+        if format == 'pmo':
+            raise pymol.CmdException('pmo format not supported anymore')
+
+        if format not in ('pdb', 'cif'):
+            raise pymol.CmdException(format + ' format not supported with multisave')
+
+        s = get_str(format, pattern, state, '', -1, 1, quiet, _self)
+
+        if s is None:
+            if _self._raising(): raise QuietException
+            return DEFAULT_ERROR
+
+        filename = _self.exp_path(filename)
+
+        with open(filename, 'a' if int(append) else 'w') as handle:
+            handle.write(s)
+
+        return DEFAULT_SUCCESS
 
     def assign_atom_types( selection, format = "mol2", state=1, quiet=1, _self=cmd):
         r = DEFAULT_ERROR
@@ -546,6 +642,111 @@ DESCRIPTION
             _self.unlock(r,_self)
         return r
 
+    def get_str(format, selection='(all)', state=-1, ref='',
+             ref_state=-1, multi=-1, quiet=1, _self=cmd):
+        '''
+DESCRIPTION
+
+    API-only function which exports the selection to a molecular file
+    format and returns it as a string.
+
+ARGUMENTS
+
+    format = str: pdb|cif|sdf|mol|mol2|mae|pqr|xyz
+
+    selection = str: atom selection {default: all}
+
+    state = int: object state (-1=current, 0=all) {default: -1}
+
+    ref = str: object name which defines reference frame {default: }
+
+    ref_state = int: state of ref object {default: -1}
+
+    multi = int: for multi-entry file formats, 0 = single entry,
+    1 = by object, 2 = by object-state, -1 = format default {default: -1}
+        '''
+        with _self.lockcm:
+            return _cmd.get_str(_self._COb, str(format), str(selection),
+                    int(state) - 1, str(ref), int(ref_state),
+                    int(multi), int(quiet))
+
+
+    def multifilesave(filename, selection='*', state=-1, format='', ref='',
+             ref_state=-1, quiet=1, _self=cmd):
+        '''
+DESCRIPTION
+
+    For a selection that spans multiple molecular objects and/or states,
+    save each object and/or state to a separate file. Takes a filename
+    argument with placeholders:
+
+    {name}  : object name
+    {state} : state number
+    {title} : state title
+    {num}   : file number
+    {}      : object name (first) or state (second)
+
+EXAMPLES
+
+    multifilesave /tmp/{name}.pdb
+    multifilesave /tmp/{name}-{state}.cif, state=0
+    multifilesave /tmp/{}-{}.cif, state=0
+    multifilesave /tmp/{}-{title}.sdf, state=0
+        '''
+        for (fname, osele, ostate) in multifilenamegen(
+                filename, selection, int(state), _self):
+            r = _self.save(fname, osele, ostate, format, ref, ref_state, quiet)
+        return r
+
+
+    def multifilenamegen(filename, selection, state, _self=cmd):
+        '''Given a filename pattern, atom selection and state argument,
+        Generate object-state specific filenames and selections.
+        '''
+        import string
+
+        filename = _self.exp_path(filename)
+
+        fmt_keys = [v[1]
+                for v in string.Formatter().parse(filename)
+                if v[1] is not None]
+
+        nindexed = fmt_keys.count('')
+        multiobject = nindexed > 0 or 'name' in fmt_keys or 'num' in fmt_keys
+        multistate = nindexed > 1 or 'state' in fmt_keys or 'title' in fmt_keys
+
+        if not (multiobject or multistate):
+            raise ValueError('need one or more of {name}, {num}, {state}, {title}')
+
+        odata = []
+        for oname in _self.get_object_list(selection):
+            osele = '(%s) & ?%s' % (selection, oname)
+            first = last = state
+
+            if multistate:
+                if state < 0:
+                    first = last = _self.get_object_state(oname)
+
+                if first == 0:
+                    first = 1
+                    last = _self.count_states('%' + oname)
+
+            for ostate in range(first, last + 1):
+                odata.append((oname, osele, ostate))
+
+        # pad {state} and {num} with zeros
+        swidth = len(str(max(v[2] for v in odata)))
+        nwidth = len(str(len(odata)))
+        filename = filename.replace('{state}', '{state:0%d}' % swidth)
+        filename = filename.replace('{num}', '{num:0%d}' % nwidth)
+
+        for num, (oname, osele, ostate) in enumerate(odata, 1):
+            fname = filename.format(oname, ostate,
+                    name=oname, state=ostate, num=num,
+                    title=_self.get_title(oname, ostate))
+            yield fname, osele, ostate
+
+
     def save(filename, selection='(all)', state=-1, format='', ref='',
              ref_state=-1, quiet=1, partial=0,_self=cmd):
         '''
@@ -589,105 +790,96 @@ SEE ALSO
 
     load, get_model
         '''
-        import gzip
-
         quiet = int(quiet)
-        do_gzip = False
 
         # preprocess selection
         selection = selector.process(selection)
         #   
         r = DEFAULT_ERROR
 
-        if format=='':
-            ext_list = filename.lower().rsplit('.', 2)
-            if ext_list[-1] == 'gz':
-                do_gzip = True
-                ext = ext_list[-2]
-            else:
-                ext = ext_list[-1]
-
-            if ext in ['cif', 'pqr', 'mol', 'sdf', 'pkl', 'xyz', 'pov',
-                    'png', 'aln', 'fasta', 'obj', 'mtl', 'wrl', 'dae', 'idtf',
-                    'mol2']:
-                format = ext
-            elif ext in ["pdb", "ent"]:
-                format = 'pdb'
-            elif ext in ["mmod", "mmd", "out", "dat"]:
-                format = 'mmod'
-            elif ext in ["pse", "psw"]:
-                format = 'pse'
-            elif ext in ["pze", "pzw"]:
-                do_gzip = True
-                format = 'pse'
-            else:
-                if not quiet:
-                    print(" Save-Warning: Unrecognized file type -- defaulting to PDB format.")
-                format='pdb'
-
+        # analyze filename
+        from pymol.importing import filename_to_format, _eval_func
+        _, _, format_guessed, zipped = filename_to_format(filename)
         filename = _self.exp_path(filename)
 
-        func_type1 = {
-            'cif': get_cifstr, # mmCIF
-            'xyz': get_xyzstr,
-            'fasta': get_fastastr,
-            'aln': get_alnstr,
-        }
+        # file format
+        if not format:
+            if not format_guessed:
+                raise pymol.CmdException('Unrecognized file format')
+            format = format_guessed
 
-        func_type2 = {
-            'pdb': get_pdbstr,
-            'pqr': get_pqrstr,
-            'sdf': get_sdfstr,
-            'mol2': get_mol2str,
-        }
-
-        def safe_getitem(tup, idx):
-            return tup[idx] if tup else None
-
-        func_type3 = {
-            'dae': _self.get_collada,
-            'wrl': _self.get_vrml,
-            'mtl': lambda: safe_getitem(_self.get_mtl_obj(), 0),
-            'obj': lambda: safe_getitem(_self.get_mtl_obj(), 1),
-            'pov': _self.get_povray,
-            'idtf': _self.get_idtf,
-        }
+        # PyMOL session
+        if format in ('pse', 'psw',):
+            _self.set("session_file",
+                    # always use unix-like path separators
+                    filename.replace("\\", "/"), quiet=1)
+            if not quiet:
+                print(" Save: Please wait -- writing session file...")
 
         func_type4 = {
             'mmod': io.mmd.toFile,
             'pkl': io.pkl.toFile, # binary pickle
             'pkla': lambda model, filename: io.pkl.toFile(model, filename, bin=0), # ascii pickle
-            'mol': io.mol.toFile,
         }
 
         contents = None
 
-        if format in func_type1:
-            contents = func_type1[format](selection, state, quiet, _self=_self)
-        elif format in func_type2:
-            contents = func_type2[format](selection, state, ref, ref_state, quiet, _self=_self)
-        elif format in func_type3:
-            contents = func_type3[format]()
-            if isinstance(contents, tuple):
-                contents = ''.join(contents)
+        if format in savefunctions:
+            # generic forwarding to format specific save functions
+            func = savefunctions[format]
+            func = _eval_func(func)
+            kw = {
+                'filename': filename,
+                'selection': selection,
+                'name': selection,      # alt (get_ccp4str)
+                'state': state,
+                'format': format,
+                'ref': ref,
+                'ref_state': ref_state,
+                'quiet': quiet,
+                'partial': partial,
+                '_self': _self,
+            }
+
+            import inspect
+            spec = inspect.getargspec(func)
+
+            if spec.varargs:
+                print('FIXME: savefunctions[%s]: *args' % (format))
+
+            if not spec.keywords:
+                kw = dict((n, kw[n]) for n in spec.args if n in kw)
+
+            contents = func(**kw)
+
+            if 'filename' in spec.args:
+                # assume function wrote directly to file and returned a status
+                return contents
+
         elif format in func_type4:
             func_type4[format](_self.get_model(selection, state, ref, ref_state), filename)
             r = DEFAULT_SUCCESS
-        elif format=='pse': # PyMOL session
-            filename = filename.replace("\\","/") # always use unix-like path separators	
-            _self.set("session_file",filename,quiet=1)
-            if '(' in selection: # ignore selections
-                selection = ''
-            if not quiet:
-                print(" Save: Please wait -- writing session file...")
-            contents = cPickle.dumps(_self.get_session(selection, partial, quiet), 1)
-        elif format=='png':
-            return _self.png(filename, quiet=quiet)
+        else:
+            raise pymol.CmdException('File format not supported for export')
+
+        # function returned sequence of strings or bytes
+        if isinstance(contents, (tuple, list)) and contents:
+            contents = contents[0][:0].join(contents)
 
         if cmd.is_string(contents):
             if not isinstance(contents, bytes):
                 contents = contents.encode()
-            with (gzip.open if do_gzip else open)(filename, 'wb') as handle:
+
+            if zipped == 'gz':
+                import gzip
+                fopen = gzip.open
+            else:
+                fopen = open
+                if zipped == 'bz2':
+                    import bz2
+                    contents = bz2.compress(contents)
+
+            with fopen(filename, 'wb') as handle:
                 handle.write(contents)
             r = DEFAULT_SUCCESS
             
@@ -701,32 +893,6 @@ SEE ALSO
 
         return r
 
-    # mmCIF export
-
-    re_cifsimpledatavalue_match = re.compile(r'[^_#\$\'"\[\];]\S*$').match
-    re_cifendsinglequote_search = re.compile(r"'\s").search
-    re_cifenddoublequote_search = re.compile(r'"\s').search
-
-    def cifisreserved(s):
-        '''return true if s is a reserved cif keyword'''
-        return s in ('loop_', 'stop_', 'global_') or s[:5] in ('data_', 'save_')
-
-    def cifrepr(s):
-        '''returns s, if s is a simple data value, or some quoted version of s'''
-        if not s:
-            return '.'
-        if not cifisreserved(s) and re_cifsimpledatavalue_match(s) is not None:
-            return s
-        if '\n' not in s:
-            if re_cifendsinglequote_search(s) is None:
-                return "'" + s + "'"
-            if re_cifenddoublequote_search(s) is None:
-                return '"' + s + '"'
-        if '\n;' in s:
-            print(' Warning: CIF data value contains <newline><semicolon>')
-            s = s.replace('\n;', '\n ;')
-        return '\n;' + s + '\n;'
-
     def get_cifstr(selection="all", state=-1, quiet=1, _self=cmd):
         '''
 DESCRIPTION
@@ -737,140 +903,16 @@ SEE ALSO
 
     get_pdbstr
         '''
-        tmp = _self.get_unused_name('_sele')
-        _self.select(tmp, selection, 0)
-
-        buf = ['# generated by PyMOL %s' % (_self.get_version()[0],)]
-
-        def callback(type_, ID, elem, name, alt, resn, segi, chain, resi,
-                x, y, z, q, b, formal_charge, state, entity_id):
-            resv, ins = (resi[:-1], resi[-1]) if resi[-1].isalpha() else (resi, '')
-            buf.append('%-6s %-3d %s %-3s '
-                    '%s %-3s %s %s '
-                    '%-2s %s %6.3f %6.3f %6.3f '
-                    '%4.2f %6.2f %d %s %d\n' % (
-                type_, ID, cifrepr(elem), cifrepr(name),
-                cifrepr(alt), cifrepr(resn), cifrepr(segi),
-                cifrepr(entity_id),
-                resv, cifrepr(ins), x, y, z,
-                q, b, formal_charge, cifrepr(chain), state,
-            ))
-
-        for model in _self.get_object_list('?' + tmp):
-            buf.append('''
-data_%s
-_entry.id %s
-''' % (model, cifrepr(model)))
-
-            symmetry = _self.get_symmetry(model)
-            if symmetry:
-                buf.append('''#
-_cell.entry_id %s
-_cell.length_a %f
-_cell.length_b %f
-_cell.length_c %f
-_cell.angle_alpha %f
-_cell.angle_beta %f
-_cell.angle_gamma %f
-_symmetry.entry_id %s
-_symmetry.space_group_name_H-M %s
-''' % (
-                    cifrepr(model),
-                    symmetry[0],
-                    symmetry[1],
-                    symmetry[2],
-                    symmetry[3],
-                    symmetry[4],
-                    symmetry[5],
-                    cifrepr(model),
-                    cifrepr(symmetry[6]),
-                ))
-
-            buf.append('''#
-loop_
-_atom_site.group_PDB
-_atom_site.id
-_atom_site.type_symbol
-_atom_site.label_atom_id
-_atom_site.label_alt_id
-_atom_site.label_comp_id
-_atom_site.label_asym_id
-_atom_site.label_entity_id
-_atom_site.label_seq_id
-_atom_site.pdbx_PDB_ins_code
-_atom_site.Cartn_x
-_atom_site.Cartn_y
-_atom_site.Cartn_z
-_atom_site.occupancy
-_atom_site.B_iso_or_equiv
-_atom_site.pdbx_formal_charge
-_atom_site.auth_asym_id
-_atom_site.pdbx_PDB_model_num
-''')
-
-            loop_i = len(buf) - 1
-
-            _self.iterate_state(state, '?%s & ?%s' % (model, tmp),
-                'callback(type, ID, elem, name, alt, resn, segi, chain, resi, '
-                'x, y, z, q, b, formal_charge, state, "")',
-                space={'callback': callback})
-
-            # no loop header for zero rows
-            if loop_i == len(buf) - 1:
-                buf[loop_i] = '#'
-
-        _self.delete(tmp)
-
-        return ''.join(buf)
+        return get_str('cif', selection, state, '', -1, -1, quiet, _self)
 
     def get_xyzstr(selection, state=-1, quiet=1, _self=cmd):
-        state = int(state)
-        buf = []
-
-        for i, (osele, ostate) in enumerate(
-                pymol.selecting.objsele_state_iter(selection, state)):
-            n_atoms_i = len(buf)
-            buf.append('') # natoms (deferred)
-            buf.append('\n') # comment
-            n = _self.iterate_state(ostate, osele,
-                    r'_buf.append("%s %f %f %f\n" % (elem, x, y, z))',
-                    space={'_buf': buf})
-            buf[n_atoms_i] = str(n) + '\n'
-
-        if not quiet:
-            print(" Save-XYZ: %d object-state(s) in selection." % (i + 1))
-
-        return ''.join(buf)
+        return get_str('xyz', selection, state, '', -1, -1, quiet, _self)
 
     def get_sdfstr(selection, state=-1, ref='', ref_state=-1, quiet=1, _self=cmd):
-        state = int(state)
-        buf = []
-
-        for i, (osele, ostate) in enumerate(
-                pymol.selecting.objsele_state_iter(selection, state)):
-            rec = SDFRec(io.mol.toList(_self.get_model(osele, ostate, ref, ref_state)))
-            buf.extend(rec.toList())
-            buf.append('$$$$\n')
-
-        if not quiet:
-            print(" Save-SDF: %d object-state(s) in selection." % (i + 1))
-
-        return ''.join(buf)
+        return get_str('sdf', selection, state, ref, ref_state, -1, quiet, _self)
 
     def get_mol2str(selection, state=-1, ref='', ref_state=-1, quiet=1, _self=cmd):
-        state = int(state)
-        buf = []
-
-        for i, (osele, ostate) in enumerate(
-                pymol.selecting.objsele_state_iter(selection, state)):
-            assign_atom_types(osele, "mol2", ostate, 1, _self)
-            buf.extend(io.mol2.toList(_self.get_model(osele,
-                ostate, ref, ref_state), selection=osele, state=ostate))
-
-        if not quiet:
-            print(" Save-MOL2: %d object-state(s) in selection." % (i + 1))
-
-        return ''.join(buf)
+        return get_str('mol2', selection, state, ref, ref_state, -1, quiet, _self)
 
     def get_alnstr(selection, state=-1, quiet=1, _self=cmd):
         with _self.lockcm:
@@ -878,6 +920,54 @@ _atom_site.pdbx_PDB_model_num
                     int(state)-1, 0, int(quiet))
 
     def get_pqrstr(selection, state=-1, ref='', ref_state=-1, quiet=1, _self=cmd):
+        return get_str('pqr', selection, state, ref, ref_state, -1, quiet, _self)
+
+    def get_maestr(selection, state=-1, ref='', ref_state=-1, quiet=1, _self=cmd):
+        return get_str('mae', selection, state, ref, ref_state, -1, quiet, _self)
+
+    def get_ccp4str(name, state=1, quiet=1, _self=cmd):
         with _self.lockcm:
-            return _cmd.get_pdb(_self._COb, str(selection), int(state)-1, 1,
-                    str(ref), int(ref_state)-1, int(quiet))
+            return _cmd.get_ccp4str(_self._COb, str(name),
+                    int(state) - 1, int(quiet))
+
+    def get_psestr(selection, partial, quiet, _self):
+        if '(' in selection: # ignore selections
+            selection = ''
+        session = _self.get_session(selection, partial, quiet)
+        return cPickle.dumps(session, 1)
+
+    def _get_mtl_obj(format, _self):
+        # TODO mtl not implemented, always returns empty string
+        if format == 'mtl':
+            raise pymol.CmdException('.MTL export not implemented')
+        i = {'mtl': 0, 'obj': 1}.get(format)
+        return _self.get_mtl_obj()[i]
+
+    savefunctions = {
+        'cif': get_str, # mmCIF
+        'xyz': get_str,
+        'pdb': get_str,
+        'pqr': get_str,
+        'sdf': get_str,
+        'mol2': get_str,
+        'mae': get_str,
+        'mol': get_str,
+        'mmtf': 'pymol.lazyio:get_mmtfstr',
+
+        'pse': get_psestr,
+        'psw': get_psestr,
+
+        'fasta': get_fastastr,
+        'aln': get_alnstr,
+        'ccp4': get_ccp4str,
+
+        'png': png,
+
+        # no arguments (some have a "version" argument)
+        'dae': 'pymol.querying:get_collada',
+        'wrl': 'pymol.querying:get_vrml',
+        'pov': 'pymol.querying:get_povray',
+        'idtf': 'pymol.querying:get_idtf',
+        'mtl': _get_mtl_obj, # TODO not implemented
+        'obj': _get_mtl_obj,
+    }
