@@ -111,12 +111,12 @@ SEE ALSO
                                 print("RMSD %f over %i residues" % (float(RMSD), int(aliLen)))
 
                         if int(transform):
-                            for model in cmd.get_object_list("(" + mobile + ")"):
+                            for model in _self.get_object_list("(" + mobile + ")"):
                                 _self.transform_object(model, rotMat, state=0)
 
                         if object is not None:
-                            obj1 = cmd.get_object_list("(" + target + ")")
-                            obj2 = cmd.get_object_list("(" + mobile + ")")
+                            obj1 = _self.get_object_list("(" + target + ")")
+                            obj2 = _self.get_object_list("(" + mobile + ")")
                             if len(obj1) > 1 or len(obj2) > 1:
                                 print(' CEalign-Error: selection spans multiple' + \
                                         ' objects, cannot create alignment object')
@@ -183,9 +183,13 @@ SEE ALSO
                         target='?%s & ?%s' % (sele_name, reference), **kwargs)
                 if not quiet:
                     if _self.is_sequence(x):
-                        print('%-20s RMS = %8.3f (%d atoms)' % (model, x[0], x[1]))
+                        print('%-20s RMSD = %8.3f (%d atoms)' % (model, x[0], x[1]))
                     elif isinstance(x, float):
-                        print('%-20s RMS = %8.3f' % (model, x))
+                        print('%-20s RMSD = %8.3f' % (model, x))
+                    elif isinstance(x, dict) and 'RMSD' in x:
+                        natoms = x.get('alignment_length', 0)
+                        suffix = (' (%s atoms)' % natoms) if natoms else ''
+                        print('%-20s RMSD = %8.3f' % (model, x['RMSD']) + suffix)
                     else:
                         print('%-20s' % (model,))
 
@@ -223,7 +227,7 @@ SEE ALSO
         extra_fit, align, super, cealign, fit, rms, rms_cur, intra_fit
                 """
                 if not selection:
-                    names = cmd.get_names("public_objects", 1)
+                    names = _self.get_names("public_objects", 1)
                     if not names:
                         raise pymol.CmdException('no public objects')
                     selection = '%' + ' %'.join(names)
